"""
File System
"""

# Copyright © 2024- Frello Technology Private Limited

import os
import re
import sys
from typing import List
from importlib.util import spec_from_file_location, module_from_spec

from tuneapi.utils.randomness import get_random_string


def get_files_in_folder(
    folder,
    ext="*",
    ig_pat: str = "",
    abs_path: bool = True,
    followlinks: bool = False,
) -> List[str]:
    """Get files with `ext` in `folder`"""
    # this method is faster than glob
    all_paths = []
    ext = [ext] if isinstance(ext, str) else ext
    _all = "*" in ext  # wildcard means everything so speed up
    ignore_pat = re.compile(ig_pat)

    folder_abs = os.path.abspath(folder) if abs_path else folder
    for root, _, files in os.walk(folder_abs, followlinks=followlinks):
        if _all:
            for f in files:
                _fp = joinp(root, f)
                if not ignore_pat.search(_fp):
                    all_paths.append(_fp)
            continue

        for f in files:
            for e in ext:
                if f.endswith(e):
                    _fp = joinp(root, f)
                    if not ignore_pat.search(_fp):
                        all_paths.append(_fp)
    return all_paths


def folder(x: str) -> str:
    """get the folder of this file path"""
    return os.path.split(os.path.abspath(x))[0]


def joinp(x: str, *args) -> str:
    """convienience function for os.path.join"""
    return os.path.join(x, *args)


def load_module_from_path(fn_name, file_path):
    spec = spec_from_file_location(fn_name, file_path)
    foo = module_from_spec(spec)
    mod_name = f"{fn_name}_{get_random_string(3)}"
    sys.modules[mod_name] = foo
    spec.loader.exec_module(foo)
    fn = getattr(foo, fn_name)
    del sys.modules[mod_name]
    return fn
